@@ -1,7 +1,7 @@
 /*
  * aurora - Communications with Magnetek Aurora Inverter
  *
- * Copyright (C) 2006-2015 Curtis J. Blank curt@curtronics.com
+ * Copyright (C) 2006-2016 Curtis J. Blank curt@curtronics.com
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -86,10 +86,12 @@
  * modified 16-nov-2013 cjb	1. handle missing tm_gmtoff in tm structure
  * modified 05-dec-2013 cjb	1. temperatures can be below zero i.e. negative (inverters installed outdoors)
  * modified 11-apr-2014 cjb	1. correct reporting wrong model when -Q is not used 
+ * modified 11-jul-2016 cjb	1. if returning _ERROR_ and veborse print out debug info
+ *				2. change how _ERROR_ is determined in float routines
  *
  */
 
-char     VersionC[7] = "1.8.8";
+char     VersionC[7] = "1.9.3";
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -127,7 +129,7 @@ static BOOL CmdNotImp = FALSE;
 static long int getGMToffset(time_t seconds);
 static int TransState(int TransCode, char Desription[], BOOL FailIf);
 static int PrintModel(void);
-static long GetCEdata(int fdser, int yAddress, int opcode, int param, char description[]);
+static unsigned long GetCEdata(int fdser, int yAddress, int opcode, int param, char description[]);
 static float GetCECdata(int fdser, int yAddress, int opcode, int param, char description[], int yGetEnergyCen);
 static float GetDSPdata(int fdser, int yAddress, int opcode, int param, char description[], BOOL TScheck);
 static time_t GetInvTime(int fdser, int yAddress);
@@ -138,7 +140,7 @@ static int ReadNextChar(int nfd, char *pChar, int timeout);
 static int ReadToBuffer(int nfd, char *pszBuffer, int nBufSize);
 static WORD crc16(char *data_p, unsigned short length);
 static char* FindString(WORD wRule, char *ptr);
-static float *szCvrtFloat(char *Buffer);
+static float szCvrtFloat(char *Buffer);
 static unsigned short szCvrtShort(char *Buffer);
 static unsigned long szCvrtLong(char *Buffer);
 static unsigned long cvrtLong(char *Buffer);
@@ -974,11 +976,12 @@ float GetCECdata(int fdser, int yAddress, int opcode, int param, char descriptio
     szSerBuffer[cParam4End] = '\0';
     nCnt = Communicate(fdser, yAddress);
     if (nCnt > 0 && TransState((int)szSerBuffer[aState],description,TRUE)) {
-        paramValFloat = *szCvrtFloat(szSerBuffer);
+        paramValFloat = szCvrtFloat(szSerBuffer);
         if (bVerbose) fprintf(stderr, "value     %12.6f\n",paramValFloat);
         return(paramValFloat);
     }
 
+    if (bVerbose) fprintf(stderr, "_ERROR_ (%i %i)\n",invOp,invFunc);
     return(_ERROR_);
 
 }
@@ -989,17 +992,17 @@ float GetCECdata(int fdser, int yAddress, int opcode, int param, char descriptio
 ----------------------------------------------------------------------------*/
 int GetCE(int fdser, int yAddress)
 {
-    long DAILY, WEEKLY, LAST7DAYS, MONTHLY, YEARLY, TOTAL, PARTIAL;
+    unsigned long DAILY, WEEKLY, LAST7DAYS, MONTHLY, YEARLY, TOTAL, PARTIAL;
 
     if (bVerbose) fprintf(stderr, "\nAttempting to get Partial Energy value ");
-    if ((PARTIAL = GetCEdata(fdser,yAddress,opGetCE,CEpar6,_opGetCE)) < 0) return(-1);
-    if ((DAILY = GetCEdata(fdser,yAddress,opGetCE,CEpar0,_opGetCE)) < 0) return(-1);
-    if ((WEEKLY = GetCEdata(fdser,yAddress,opGetCE,CEpar1,_opGetCE)) < 0) return(-1);
+    if ((PARTIAL = GetCEdata(fdser,yAddress,opGetCE,CEpar6,_opGetCE)) == _ERROR_) return(-1);
+    if ((DAILY = GetCEdata(fdser,yAddress,opGetCE,CEpar0,_opGetCE)) == _ERROR_) return(-1);
+    if ((WEEKLY = GetCEdata(fdser,yAddress,opGetCE,CEpar1,_opGetCE)) == _ERROR_) return(-1);
     LAST7DAYS = 0.0;    /* do this for now since this has been dropped in the v2.3 Communications Protocol doc  */
                         /* placeholder for the -c option (colums)for now                                                */
-    if ((MONTHLY = GetCEdata(fdser,yAddress,opGetCE,CEpar3,_opGetCE)) < 0) return(-1);
-    if ((YEARLY = GetCEdata(fdser,yAddress,opGetCE,CEpar4,_opGetCE)) < 0) return(-1);
-    if ((TOTAL = GetCEdata(fdser,yAddress,opGetCE,CEpar5,_opGetCE)) < 0) return(-1);
+    if ((MONTHLY = GetCEdata(fdser,yAddress,opGetCE,CEpar3,_opGetCE)) == _ERROR_) return(-1);
+    if ((YEARLY = GetCEdata(fdser,yAddress,opGetCE,CEpar4,_opGetCE)) == _ERROR_) return(-1);
+    if ((TOTAL = GetCEdata(fdser,yAddress,opGetCE,CEpar5,_opGetCE)) == _ERROR_) return(-1);
 
     if (bColumns) {
         /* must continue to output LAST7DAYS as a placeholder even though the inverter(s) may no longer report it */
@@ -1036,7 +1039,7 @@ int GetCE(int fdser, int yAddress)
 /*--------------------------------------------------------------------------
     GetCEdata
 ----------------------------------------------------------------------------*/
-long GetCEdata(int fdser, int yAddress, int opcode, int param, char description[])
+unsigned long GetCEdata(int fdser, int yAddress, int opcode, int param, char description[])
 {
     int nCnt;
     unsigned long paramValLong;
@@ -1059,7 +1062,8 @@ long GetCEdata(int fdser, int yAddress, int opcode, int param, char description[
         return((int)paramValLong);
     }
 
-    return(-1);
+    if (bVerbose) fprintf(stderr, "_ERROR_ (%i %i)\n",invOp,invFunc);
+    return(_ERROR_);
 }
 
 
@@ -1073,18 +1077,18 @@ int GetDSP(int fdser, int yAddress)
  
     STR1V = STR1C = STR2V = STR2C = -1.0;
 
-    if ((FRQ = GetDSPdata(fdser,yAddress,opGetDSP,ToM4,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-    if ((GVR = GetDSPdata(fdser,yAddress,opGetDSP,ToM1,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-    if ((GCR = GetDSPdata(fdser,yAddress,opGetDSP,ToM2,_opGetDSP,TRUE)) < -1000000.0) return(-1);
-    if ((GPR = GetDSPdata(fdser,yAddress,opGetDSP,ToM3,_opGetDSP,TRUE)) < -1000000.0) return(-1);
+    if ((FRQ = GetDSPdata(fdser,yAddress,opGetDSP,ToM4,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+    if ((GVR = GetDSPdata(fdser,yAddress,opGetDSP,ToM1,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+    if ((GCR = GetDSPdata(fdser,yAddress,opGetDSP,ToM2,_opGetDSP,TRUE)) >= _ERROR_) return(-1);	// < -1000000.0
+    if ((GPR = GetDSPdata(fdser,yAddress,opGetDSP,ToM3,_opGetDSP,TRUE)) >= _ERROR_) return(-1);	// < -1000000.0
 
     if (yGetDSP == 0 || yGetDSP == 1) {
-        if ((STR1V = GetDSPdata(fdser,yAddress,opGetDSP,ToM23,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((STR1C = GetDSPdata(fdser,yAddress,opGetDSP,ToM25,_opGetDSP,TRUE)) == _ERROR_) return(-1);
+        if ((STR1V = GetDSPdata(fdser,yAddress,opGetDSP,ToM23,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((STR1C = GetDSPdata(fdser,yAddress,opGetDSP,ToM25,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
     }
     if (yCentral < 0 && (yGetDSP == 0 || yGetDSP == 2)) {
-        if ((STR2V = GetDSPdata(fdser,yAddress,opGetDSP,ToM26,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((STR2C = GetDSPdata(fdser,yAddress,opGetDSP,ToM27,_opGetDSP,TRUE)) == _ERROR_) return(-1);
+        if ((STR2V = GetDSPdata(fdser,yAddress,opGetDSP,ToM26,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((STR2C = GetDSPdata(fdser,yAddress,opGetDSP,ToM27,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
     }
 
     if (yCentral < 0) {
@@ -1190,39 +1194,39 @@ int GetDSPExtended(int fdser, int yAddress)
     VB = ILD = ILI = P1 = P2 = GV = GF = IR = VBD = AGV = VBM = PP = PPT = GVn = WGF = 0.0;
     VBp = VBm = ST = AT = HT = T1 = T2 = T3 = F1 = F2 = F3 = F4 = F5 = PSL = RRB = VPm = 0.0;
 
-    if ((VB = GetDSPdata(fdser,yAddress,opGetDSP,ToM5,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-    if ((IR = GetDSPdata(fdser,yAddress,opGetDSP,ToM30,_opGetDSP,FALSE)) == _ERROR_) return(-1);
-    if ((P1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM8,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-    if ((PPT = GetDSPdata(fdser,yAddress,opGetDSP,ToM35,_opGetDSP,TRUE)) == _ERROR_) return(-1);
+    if ((VB = GetDSPdata(fdser,yAddress,opGetDSP,ToM5,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+    if ((IR = GetDSPdata(fdser,yAddress,opGetDSP,ToM30,_opGetDSP,FALSE)) >= _ERROR_) return(-1);
+    if ((P1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM8,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+    if ((PPT = GetDSPdata(fdser,yAddress,opGetDSP,ToM35,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
     if (yCentral < 0) {
-        if ((ILD = GetDSPdata(fdser,yAddress,opGetDSP,ToM6,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((ILI = GetDSPdata(fdser,yAddress,opGetDSP,ToM7,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((P2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM9,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((GV = GetDSPdata(fdser,yAddress,opGetDSP,ToM28,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((GF = GetDSPdata(fdser,yAddress,opGetDSP,ToM29,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((VBD = GetDSPdata(fdser,yAddress,opGetDSP,ToM31,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((AGV = GetDSPdata(fdser,yAddress,opGetDSP,ToM32,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((VBM = GetDSPdata(fdser,yAddress,opGetDSP,ToM33,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((PP = GetDSPdata(fdser,yAddress,opGetDSP,ToM34,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((GVn = GetDSPdata(fdser,yAddress,opGetDSP,ToM36,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((WGF = GetDSPdata(fdser,yAddress,opGetDSP,ToM37,_opGetDSP,TRUE)) == _ERROR_) return(-1);
+        if ((ILD = GetDSPdata(fdser,yAddress,opGetDSP,ToM6,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((ILI = GetDSPdata(fdser,yAddress,opGetDSP,ToM7,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((P2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM9,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((GV = GetDSPdata(fdser,yAddress,opGetDSP,ToM28,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((GF = GetDSPdata(fdser,yAddress,opGetDSP,ToM29,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((VBD = GetDSPdata(fdser,yAddress,opGetDSP,ToM31,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((AGV = GetDSPdata(fdser,yAddress,opGetDSP,ToM32,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((VBM = GetDSPdata(fdser,yAddress,opGetDSP,ToM33,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((PP = GetDSPdata(fdser,yAddress,opGetDSP,ToM34,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((GVn = GetDSPdata(fdser,yAddress,opGetDSP,ToM36,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((WGF = GetDSPdata(fdser,yAddress,opGetDSP,ToM37,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
     } else {
-        if ((VBp = GetDSPdata(fdser,yAddress,opGetDSP,ToM45,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((VBm = GetDSPdata(fdser,yAddress,opGetDSP,ToM46,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((ST = GetDSPdata(fdser,yAddress,opGetDSP,ToM47,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((AT = GetDSPdata(fdser,yAddress,opGetDSP,ToM48,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((HT = GetDSPdata(fdser,yAddress,opGetDSP,ToM49,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((T1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM50,_opGetDSP,FALSE)) == _ERROR_) return(-1);
-        if ((T2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM51,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((T3 = GetDSPdata(fdser,yAddress,opGetDSP,ToM52,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((F1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM53,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((F2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM54,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((F3 = GetDSPdata(fdser,yAddress,opGetDSP,ToM55,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((F4 = GetDSPdata(fdser,yAddress,opGetDSP,ToM56,_opGetDSP,FALSE)) == _ERROR_) return(-1);
-        if ((F5 = GetDSPdata(fdser,yAddress,opGetDSP,ToM57,_opGetDSP,FALSE)) == _ERROR_) return(-1);
-        if ((PSL = GetDSPdata(fdser,yAddress,opGetDSP,ToM58,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((RRB = GetDSPdata(fdser,yAddress,opGetDSP,ToM59,_opGetDSP,TRUE)) == _ERROR_) return(-1);
-        if ((VPm = GetDSPdata(fdser,yAddress,opGetDSP,ToM60,_opGetDSP,TRUE)) == _ERROR_) return(-1);
+        if ((VBp = GetDSPdata(fdser,yAddress,opGetDSP,ToM45,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((VBm = GetDSPdata(fdser,yAddress,opGetDSP,ToM46,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((ST = GetDSPdata(fdser,yAddress,opGetDSP,ToM47,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((AT = GetDSPdata(fdser,yAddress,opGetDSP,ToM48,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((HT = GetDSPdata(fdser,yAddress,opGetDSP,ToM49,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((T1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM50,_opGetDSP,FALSE)) >= _ERROR_) return(-1);
+        if ((T2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM51,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((T3 = GetDSPdata(fdser,yAddress,opGetDSP,ToM52,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((F1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM53,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((F2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM54,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((F3 = GetDSPdata(fdser,yAddress,opGetDSP,ToM55,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((F4 = GetDSPdata(fdser,yAddress,opGetDSP,ToM56,_opGetDSP,FALSE)) >= _ERROR_) return(-1);
+        if ((F5 = GetDSPdata(fdser,yAddress,opGetDSP,ToM57,_opGetDSP,FALSE)) >= _ERROR_) return(-1);
+        if ((PSL = GetDSPdata(fdser,yAddress,opGetDSP,ToM58,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((RRB = GetDSPdata(fdser,yAddress,opGetDSP,ToM59,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
+        if ((VPm = GetDSPdata(fdser,yAddress,opGetDSP,ToM60,_opGetDSP,TRUE)) >= _ERROR_) return(-1);
     }
 
     note[0] = '\0';
@@ -1343,11 +1347,12 @@ float GetDSPdata(int fdser, int yAddress, int opcode, int param, char descriptio
     szSerBuffer[cParam1End] = '\0';
     nCnt = Communicate(fdser, yAddress);
     if (nCnt > 0 && (! TScheck || TransState((int)szSerBuffer[aState],description,TScheck))) {
-        if (! CmdNotImp)  paramValFloat = *szCvrtFloat(szSerBuffer);
+        if (! CmdNotImp)  paramValFloat = szCvrtFloat(szSerBuffer);
         if (bVerbose) fprintf(stderr, "value     %12.6f\n",paramValFloat);
         return(paramValFloat);
     }
 
+    if (bVerbose) fprintf(stderr, "_ERROR_ (%i %i)\n",invOp,invFunc);
     return(_ERROR_);
 }
 
@@ -1816,7 +1821,7 @@ int Communicate(int fdser, int yAddress)
         attempts++;
     }
     if (CRCrc < 0) return(-1);
-    if (bRptReties) {
+    if (bRptRetries) {
         fprintf(stderr, "\n%s: %s: %i attempts made",getCurTime(),ProgramName,attempts-1);
         if (bVerbose) fprintf(stderr, "\n");
     } else
@@ -1828,7 +1833,7 @@ int Communicate(int fdser, int yAddress)
     szCvrtFloat
     Converts a 4 char string to a float.
 ----------------------------------------------------------------------------*/
-float *szCvrtFloat(char *Buffer)
+float szCvrtFloat(char *Buffer)
 {
     unsigned char cValue[4];
     float *value;
@@ -1848,7 +1853,7 @@ float *szCvrtFloat(char *Buffer)
     value = (float *)cValue;
     if (bVerbose) fprintf(stderr, "szCvrtFloat %12.6f 0x%02x%02x%02x%02x\n",*value,cValue[3],cValue[2],cValue[1],cValue[0]);
 
-    return(value);
+    return(*value);
 }
 
 /*--------------------------------------------------------------------------
