@@ -1,7 +1,7 @@
 /*
  * aurora - Communications with Magnetek Aurora Inverter
  *
- * Copyright (C) 2006-2015 Curtis J. Blank curt@curtronics.com
+ * Copyright (C) 2006-2016 Curtis J. Blank curt@curtronics.com
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
  *
  */
 
-#define	VersionMH	"1.8.7"
+#define	VersionMH	"1.9.1"
 
 #ifndef __MAIN_H__
 #define __MAIN_H__
@@ -41,7 +41,7 @@
 typedef int BOOL;		/* yet another bool definition */
 #define TRUE 1
 #define FALSE 0
-#define _ERROR_ -999999999
+#define _ERROR_ 0xffffffff
 
 typedef unsigned char BYTE;
 typedef unsigned short WORD;
@@ -62,7 +62,7 @@ extern BOOL bGetEnergy;
 extern BOOL bCommCheck;
 extern BOOL bColOutput;
 extern BOOL bCalcGridPwr;
-extern BOOL bRptReties;
+extern BOOL bRptRetries;
 extern BOOL bRptReadPause;
 extern BOOL bSwapEndian;
 extern BOOL bUseMultiplierQ;
@@ -89,6 +89,8 @@ extern int ClrSerLock(long unsigned int PID);
 extern int RestorePort(int fdser);
 
 #define	ttyLCKloc	"/var/lock/LCK.."	/* location and prefix of serial port lock file */
+#define uSecsLCKWaitMax    100000		/* 100 millseconds */
+#define uSecsLCKWaitMin    10000		/*  10 Millseconds */
 
 /* macros */
 #define MAX(a,b) ({ typeof (a) _a = (a); typeof (b) _b = (b); _a > _b ? _a : _b; })
