@@ -1,7 +1,7 @@
 /*
  * aurora - Communications with Magnetek Aurora Inverter
  *
- * Copyright (C) 2006-2015 Curtis J. Blank curt@curtronics.com
+ * Copyright (C) 2006-2016 Curtis J. Blank curt@curtronics.com
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -68,10 +68,13 @@
  * modified 24-feb-2012 cjb	1. modify -i, --get-count option to accept a bitmask
  * modified 25-feb-2012 cjb	1. minor changes to help
  * modified 12-jun-2012 cjb	1. make sure errno is initialized
+ * modified 18-aug-2015 cjb	1. improve device lock file handling - see RELEASENOTES
+ * modified 19-jan-2016 cjb	1. on error when checking to print invFunc 0 (zero) is inclusive.
+ * modified 20-jan-2016 cjb	1. lckCNT no longer used
  *
  */
 
-static char	VersionM[] = "1.8.8";
+static char	VersionM[] = "1.9.3";
 char     VersionC[7];
 
 #include <syscall.h>
@@ -111,7 +114,7 @@ BOOL bColOutput = FALSE;
 BOOL bCalcGridPwr = FALSE;
 BOOL bXonXoff = FALSE;
 BOOL bRTSCTS = FALSE;
-BOOL bRptReties = FALSE;
+BOOL bRptRetries = FALSE;
 BOOL bRptReadPause = FALSE;
 BOOL bSwapEndian = FALSE;
 BOOL bUseMultiplierQ = FALSE;
@@ -160,7 +163,7 @@ static BOOL bGetLastAlarms = FALSE;		/* Get last four alarms */
 static BOOL bFileError = FALSE;
 static int yGetEnergyDaily = 0;
 static int yGetEnergySent = 0;
-static int yLockWait = 0;			/* Seconds to wait to lock serial port */
+static float yLockWait = 0.0;			/* Seconds to wait to lock serial port */
 char *devLCKfile = NULL;
 char *devLCKfileNew = NULL;
 static struct termios oldtio;			/* current serial device configuration */
@@ -169,6 +172,9 @@ static struct termios oldtio;			/* current serial device configuration */
 static int GetParms(int argc, char *argv[]);
 static void *getMemPtr(size_t mSize);
 static void Version();
+static int getPIDcmdLen(long unsigned int PID);
+static void getPIDcmd(long unsigned int PID, char* COMMAND);
+static int rnduSecs();
 
 /*--------------------------------------------------------------------------
     getCurTime
@@ -187,6 +193,70 @@ char* getCurTime()
     return CurTime;
 }
 
+/*--------------------------------------------------------------------------
+    getPIDcmdLen
+----------------------------------------------------------------------------*/
+int getPIDcmdLen(long unsigned int PID)
+{
+    FILE *fdserlck = NULL;
+    int fLen = 0;
+    char sPID[10];
+    char *cmdFile = NULL;
+
+    sPID[0] = '\0';
+    sprintf(sPID,"%lu",PID);
+    cmdFile = getMemPtr(strlen(sPID)+14+1);
+    cmdFile[0] = '\0';
+    sprintf(cmdFile,"/proc/%lu/cmdline",PID);
+    cmdFile[strlen(cmdFile)] = '\0';
+    fdserlck = fopen(cmdFile, "r");
+    if (fdserlck != NULL) {
+        while (fgetc(fdserlck) != EOF) fLen++;
+        fclose(fdserlck);
+        fdserlck = NULL;
+    }
+    if (cmdFile != NULL) {
+        free(cmdFile);
+        cmdFile = NULL;
+    }
+    return fLen;
+}
+
+
+/*--------------------------------------------------------------------------
+    getPIDcmd
+----------------------------------------------------------------------------*/
+void getPIDcmd(long unsigned int PID, char* COMMAND)
+{
+    FILE *fdserlck = NULL;
+    int bRead = 0;
+    int fLen = 0;
+    char sPID[10];
+    char *cmdFile = NULL;
+
+    sPID[0] = '\0';
+    sprintf(sPID,"%lu",PID);
+    cmdFile = getMemPtr(strlen(sPID)+14+1);
+    cmdFile[0] = '\0';
+    sprintf(cmdFile,"/proc/%lu/cmdline",PID);
+    cmdFile[strlen(cmdFile)] = '\0';
+    fLen = getPIDcmdLen(PID);
+    if (fLen > 0) {
+        fdserlck = fopen(cmdFile, "r");
+        if (fdserlck != NULL) {
+            COMMAND[0] = '\0';
+            bRead = fscanf(fdserlck, "%s", COMMAND);
+            if (bRead) COMMAND[strlen(COMMAND)] = '\0';
+            fclose(fdserlck);
+            fdserlck = NULL;
+        }
+    }
+    if (cmdFile != NULL) {
+        free(cmdFile);
+        cmdFile = NULL;
+    }
+}
+
 
 /*--------------------------------------------------------------------------
     main
@@ -201,22 +271,30 @@ int main(int argc, char *argv[])
     time_t endTimeValue;
     char RunTime[18] = " ";
     char EndTime[18] = " ";
-    long unsigned int rPID;
-    char sPID[10];
-    int bRead, bWrite, lckCNT;
+    long unsigned int LckPID;
+    int bRead, bWrite;
     int errno_save = 0;
     int fLen = 0;
-    char *cmdFile = NULL;
-    char *command = NULL;
-    char *SubStrPos = NULL;
+    int curChar = 0;
+    int uSlckWait = 0;
+    float LockWaitElapsed = 0.0;
+    float LockWaitElapsedTot = 0.0;
+    BOOL lckCNTbeg = TRUE;
+    char *COMMAND = NULL;
+    char *LckCOMMAND = NULL;
+    char *LckPIDcommand = NULL;
     long L=1; 
     void *Ptr=&L; 
     char Endian=*(char*)Ptr;
+    struct timeval startv;
 
 
     lastcommtv.tv_sec = 0;
     lastcommtv.tv_usec = 0;
     startTimeValue = time(NULL);
+    gettimeofday(&startv, NULL);
+    srand((startv.tv_usec & 0x7ffe) + 1);
+
     timStruct = *(localtime(&startTimeValue));
     strftime(RunTime,sizeof(RunTime),"%Y%m%d-%H:%M:%S",&timStruct);
     RunTime[sizeof(RunTime)-1] = '\0';
@@ -307,7 +385,8 @@ int main(int argc, char *argv[])
         printf("                                  reading response from inverter (1-10000)\n");
         printf(" -u, --rpt-read-pause           Report when/that pausing before read\n");
         printf(" -W, --swap-endian              Swap Endianness\n");
-        printf(" -w <num>, --lock-wait=<num>    Seconds to wait to lock serial port. (1-30)\n");
+        printf(" -w <num>, --lock-wait=<num>    Milliseconds to wait to lock serial port. (%.3f-30000.0 mS)\n",(float)(uSecsLCKWaitMin/1000.0));
+        printf("                                  Default 0.0 mS meaning only one attempt to get the lock will be made.\n");
         printf(" -X, --rts-cts                  Enable RTS/CTS on the serial port.\n");
 	printf(" -x, --xon-xoff                 Enable XON/XOFF on the serial port.\n");
         printf(" -Y <num>, --retries=<num>      Retry failed communications with inverter up to <num> times (1-100)\n");
@@ -345,6 +424,10 @@ int main(int argc, char *argv[])
         exit(0);
     }
 
+    fLen = getPIDcmdLen(PID);
+    COMMAND = getMemPtr(fLen+1);
+    getPIDcmd(PID,COMMAND);
+
     if (bVerbose) fprintf(stderr, "\nAttempting to get lock on Serial Port %s...\n",szttyDevice);
     fdserlck = fopen((const char *)devLCKfile, "a");
     if (fdserlck == NULL) {
@@ -352,7 +435,7 @@ int main(int argc, char *argv[])
         fprintf(stderr, "%s: %s: Problem locking serial device, can't open lock file: %s for write.\n\n",getCurTime(),ProgramName,devLCKfile);
         exit(2);
     }
-    bWrite = fprintf(fdserlck, "%lu\n", PID);
+    bWrite = fprintf(fdserlck, "%lu %s\n", PID, COMMAND);
     errno_save = errno;
     fclose(fdserlck);
     fdserlck = NULL;
@@ -362,21 +445,32 @@ int main(int argc, char *argv[])
         exit(2);
     }
 
-    rPID = 0;
-    lckCNT = -1;
-    while(rPID != PID && lckCNT++ < yLockWait) {
-        if (bVerbose && lckCNT == 0) fprintf(stderr, "Checking for lock\n");
-        SubStrPos = NULL;
+    LckPID = 0;
+    lckCNTbeg = TRUE;
+    while(LckPID != PID && yLockWait >= 0.0) {
+        if (bVerbose && lckCNTbeg) {
+            fprintf(stderr, "Checking for lock\n");
+            lckCNTbeg = FALSE;
+        }
         fdserlck = fopen(devLCKfile, "r");
         if (fdserlck == NULL) {
             if (bVerbose) fprintf(stderr, "\n");
             fprintf(stderr, "%s: %s: Problem locking serial device, can't open lock file: %s for read.\n\n",getCurTime(),ProgramName,devLCKfile);
             exit(2);
         }
-        bRead = fscanf(fdserlck, "%lu", &rPID);
+
+        fLen = 0;
+        while ((curChar = fgetc(fdserlck)) != EOF && curChar != '\n' && curChar != ' ') fLen++;
+        fLen = 0;
+        while ((curChar = fgetc(fdserlck)) != EOF && curChar != '\n') fLen++;
+        rewind(fdserlck);
+        LckCOMMAND = getMemPtr(fLen+1);
+        LckCOMMAND[0] = '\0';
+
+        bRead = fscanf(fdserlck, "%lu %s", &LckPID, LckCOMMAND);
         errno_save = errno;
         fclose(fdserlck);
-        if (bVerbose) fprintf(stderr, "\nChecking process %lu for lock\n",rPID);
+        if (bVerbose && LockWaitElapsed >= 1000.0) fprintf(stderr, "\nChecking process %lu (%s) for lock\n", LckPID, LckCOMMAND);
         fdserlck = NULL;
         if (bRead == EOF || errno_save != 0) {
             if (bVerbose) fprintf(stderr, "\n");
@@ -384,54 +478,57 @@ int main(int argc, char *argv[])
             exit(2);
         }
 
-        sPID[0] = '\0';
-        sprintf(sPID,"%lu",rPID);
-        cmdFile = getMemPtr(strlen(sPID)+14+1);
-        cmdFile[0] = '\0';
-        sprintf(cmdFile,"/proc/%lu/cmdline",rPID);
-        cmdFile[strlen(cmdFile)] = '\0';
-        fdserlck = fopen(cmdFile, "r");
-        if (fdserlck != NULL) {
-            fLen = 0;
-            while (fgetc(fdserlck) != EOF) fLen++;
-            if (fLen > 0) {
-                command = getMemPtr(fLen+1);
-                command[0] = '\0';
-                rewind(fdserlck);
-                bRead = fscanf(fdserlck, "%s", command);
-                command[strlen(command)] = '\0';
-            }
-            fclose(fdserlck);
-            fdserlck = NULL;
-            if (command != NULL) SubStrPos = strstr(command, ProgramName);
-        }
-        if (cmdFile != NULL) {
-            free(cmdFile);
-            cmdFile = NULL;
-        }
+        fLen = getPIDcmdLen(LckPID);
+        LckPIDcommand = getMemPtr(fLen+1);
+        getPIDcmd(LckPID,LckPIDcommand);
+
         if (bVerbose) {
-            fprintf (stderr, "rPID: %lu SubStrPos: %s command: %s",rPID,SubStrPos,command);
-            if (rPID == PID) fprintf (stderr, " = me");
-            fprintf (stderr, "\n");
+            if (LckPID == PID)
+                fprintf (stderr, "PID: %lu COMMAND: \"%s\" LckPID: %lu LckCOMMAND: \"%s\" LckPIDcommand \"%s\" = me\n", PID, COMMAND, LckPID, LckCOMMAND, LckPIDcommand);
+            else if (LockWaitElapsed >= 1000.0)
+                fprintf (stderr, "PID: %lu COMMAND: \"%s\" LckPID: %lu LckCOMMAND: \"%s\" LckPIDcommand \"%s\"\n", PID, COMMAND, LckPID, LckCOMMAND, LckPIDcommand);
         }
-        if (command != NULL) {
-            free(command);
-            command = NULL;
+
+//      PID           - this process
+//      LckPID        - PID from lock file
+//	COMMAND       - this process
+//	LckCOMMAND    - process command from lock file
+//	LckPIDcommand - process command of process using PID from lock file
+        if ((PID != LckPID && LckPIDcommand == NULL) || strcmp(LckCOMMAND,LckPIDcommand) != 0 || strcmp(LckPIDcommand,"") == 0) {
+            if (bVerbose && uSlckWait != 0) fprintf (stderr, "\n");
+            fprintf(stderr, "%s: %s: Clearing stale serial port lock. (%lu)\n",getCurTime(),ProgramName,LckPID);
+            ClrSerLock(LckPID);
+            LckPID = 0;
         }
-        if (rPID != PID) {
-             if (SubStrPos == NULL) {
-                 if (bVerbose) fprintf (stderr, "\n");
-                 fprintf(stderr, "%s: %s: Clearing stale serial port lock. (%lu)\n",getCurTime(),ProgramName,rPID);
-                 ClrSerLock(rPID);
-             } else if (yLockWait > 0)
-                 sleep(1);
+        if (PID != LckPID && LckPID != 0 && yLockWait >= 0.0) {
+            uSlckWait = rnduSecs();
+            if (bVerbose && LockWaitElapsed >= 1000.0) {
+                fprintf (stderr, "Waited: %.3f mS\n",LockWaitElapsed); 
+                LockWaitElapsed = 0.0;
+            }
+            usleep(uSlckWait);
+            yLockWait -= (float)(uSlckWait/1000);
+            LockWaitElapsed += (float)(uSlckWait/1000);
+            LockWaitElapsedTot += (float)(uSlckWait/1000);  
+        }
+        if (LckCOMMAND != NULL) {
+            free(LckCOMMAND);
+            LckCOMMAND = NULL;
+        }
+        if (LckPIDcommand != NULL) {
+            free(LckPIDcommand);
+            LckPIDcommand = NULL;
         }
     }
-    if (bVerbose && rPID == PID) fprintf(stderr, "Appears we got the lock.\n");
-    if (rPID != PID) {
-        ClrSerLock(PID);
+    free(COMMAND);
+    if (bVerbose) {
+        if (LockWaitElapsedTot > 0.0) fprintf(stderr, "\nLockWaitElapsedTot: %.3f mS\n",LockWaitElapsedTot);
+        if (LckPID == PID) fprintf(stderr, "Appears we got the lock.\n");
+    }
+    if (LckPID != PID) {
         if (bVerbose) fprintf (stderr, "\n");
-        fprintf(stderr, "%s: %s: Problem locking serial device %s, couldn't get the lock for %lu, locked by %lu.\n\n",getCurTime(),ProgramName,szttyDevice,PID,rPID);
+        fprintf(stderr, "%s: %s: Problem locking serial device %s, couldn't get the lock for %lu, locked by %lu.\n",getCurTime(),ProgramName,szttyDevice,PID,LckPID);
+        ClrSerLock(PID);
         exit(2);
     }
 
@@ -656,10 +753,10 @@ int main(int argc, char *argv[])
     ClrSerLock(PID);
 
     if (rc < 0) {
-        if (bRptReties) fprintf(stderr, "\n");
+        if (bRptRetries) fprintf(stderr, "\n");
         if (bVerbose) fprintf(stderr, "\n");
         fprintf(stderr, "%s: %s: ERROR: Received bad return code (%i %i",getCurTime(),ProgramName,rc,invOp);
-        if (invFunc > 0)
+        if (invFunc >= 0)
             fprintf(stderr, " %i)\n",invFunc);
         else
             fprintf(stderr, ")\n");
@@ -725,32 +822,51 @@ int RestorePort(int fdser) {
     ClrSerLock
     Clear Serial Port lock.
 ----------------------------------------------------------------------------*/
-int ClrSerLock(long unsigned int PID) {
+int ClrSerLock(long unsigned int LckPID) {
     FILE *fdserlck, *fdserlcknew;
-    long unsigned int rPID;
+    long unsigned int PID;
     int bWrite, bRead;
     int errno_save = 0;
+    int fLen = 0;
+    int cmdLen = 0;
+    int curChar = 0;
+    char *COMMAND = NULL;
 
     errno = 0;
-    if (bVerbose) fprintf(stderr, "\ndevLCKfile: <%s>\ndevLCKfileNew: <%s>\nClearing Serial Port Lock (%lu)...", devLCKfile, devLCKfileNew, PID);
+    if (bVerbose) fprintf(stderr, "\ndevLCKfile: <%s>\ndevLCKfileNew: <%s>\nClearing Serial Port Lock (%lu)...", devLCKfile, devLCKfileNew, LckPID);
+
     fdserlck = fopen(devLCKfile, "r");
     if (fdserlck == NULL) {
         if (bVerbose) fprintf(stderr, "\n");
-        fprintf(stderr, "\n%s: %s: Problem opening serial device lock file to clear PID %lu: %s for read.\n\n",getCurTime(),ProgramName,PID,devLCKfile);
+        fprintf(stderr, "\n%s: %s: Problem opening serial device lock file to clear LckPID %lu: %s for read.\n\n",getCurTime(),ProgramName,LckPID,devLCKfile);
         return(0);
     }
     fdserlcknew = fopen(devLCKfileNew, "w");
     if (fdserlcknew == NULL) {
         if (bVerbose) fprintf(stderr, "\n");
-        fprintf(stderr, "\n%s: %s: Problem opening new serial device lock file to clear PID %lu: %s for write.\n\n",getCurTime(),ProgramName,PID,devLCKfileNew);
+        fprintf(stderr, "\n%s: %s: Problem opening new serial device lock file to clear LckPID %lu: %s for write.\n\n",getCurTime(),ProgramName,LckPID,devLCKfileNew);
         fclose(fdserlck);
         return(0);
     }
-    bRead = fscanf(fdserlck, "%lu", &rPID);
+
+    curChar = 0;
+    while (curChar != EOF) {
+        fLen = 0;
+        while ((curChar = fgetc(fdserlck)) != EOF && curChar != '\n' && curChar != ' ') fLen++;
+        fLen = 0;
+        while ((curChar = fgetc(fdserlck)) != EOF && curChar != '\n') fLen++;
+        if (fLen > cmdLen) cmdLen = fLen;
+    }
+    rewind(fdserlck);
+
+    COMMAND = getMemPtr(cmdLen+1);
+    COMMAND[0] = '\0';
+    bRead = fscanf(fdserlck, "%lu %s", &PID, COMMAND);
+
     while (bRead != EOF) {
-        if (rPID != PID) {
+        if (PID != LckPID) {
             errno = 0;
-            bWrite = fprintf(fdserlcknew, "%lu\n", rPID);
+            bWrite = fprintf(fdserlcknew, "%lu %s\n", PID, COMMAND);
             errno_save = errno;
             if (bWrite < 0 || errno_save != 0) {
                 fprintf(stderr, "\n%s: %s: Problem clearing serial device lock, can't write lock file: %s.\n%s\n\n",getCurTime(),ProgramName,devLCKfile,strerror (errno_save));
@@ -758,10 +874,11 @@ int ClrSerLock(long unsigned int PID) {
                 return(0);
             }
         }
-        bRead = fscanf(fdserlck, "%lu", &rPID);
+        bRead = fscanf(fdserlck, "%lu %s", &PID, COMMAND);
     }
     fclose(fdserlck);
     fclose(fdserlcknew);
+    free(COMMAND);
     errno = 0;
     if (rename(devLCKfileNew,devLCKfile)) fprintf(stderr, "\n%s: %s: Problem clearing serial device lock, can't update lock file: %s.\n%s\n\n",getCurTime(),ProgramName,devLCKfile,strerror (errno));
     if (bVerbose) fprintf(stderr, " done.\n");
@@ -1057,9 +1174,9 @@ int GetParms(int argc, char *argv[])
             case 'u': bRptReadPause	= TRUE; break;
             case 'W': bSwapEndian	= TRUE; break;
             case 'w':
-                yLockWait = atoi(optarg);
-                if (yLockWait < 1 || yLockWait > 30) {
-                    fprintf(stderr, "\n%s: %s: -w Lock Wait seconds (%d) out of range, 1-30.\n",getCurTime(),ProgramName,yLockWait);
+                yLockWait = atof(optarg);
+                if (yLockWait < (uSecsLCKWaitMin/1000.0) || yLockWait > 30000.0) {
+                    fprintf(stderr, "\n%s: %s: -w Lock Wait milliseconds (%.3f) out of range, %.3f-30000.0 mS.\n",getCurTime(),ProgramName,yLockWait,(float)(uSecsLCKWaitMin/1000.0));
                     return 0;
                 }
                 break;
@@ -1072,7 +1189,7 @@ int GetParms(int argc, char *argv[])
                     return 0;
                 }
                 break;
-            case 'y': bRptReties   = TRUE; break;
+            case 'y': bRptRetries   = TRUE; break;
             case 'V': bVersion     = TRUE; break;
             case 'v': bGetVer      = TRUE; break;
 
@@ -1127,6 +1244,24 @@ int GetParms(int argc, char *argv[])
 }
 
 
+/*--------------------------------------------------------------------------
+    rnduSecs
+    randomize lock wait time
+----------------------------------------------------------------------------*/
+int rnduSecs() {
+    float rnumf = 0.0;
+    int waitusecs = 0;
+
+    rnumf = (float)rand();
+    while (rnumf > 1) rnumf = rnumf/10.0;
+    waitusecs = uSecsLCKWaitMax * rnumf;;
+    if (waitusecs <= 1) waitusecs = uSecsLCKWaitMin;
+    while (waitusecs < uSecsLCKWaitMin) waitusecs = waitusecs * 1.29;
+
+    return(waitusecs);
+}
+
+
 /*--------------------------------------------------------------------------
         getMemPtr
 ----------------------------------------------------------------------------*/
